@@ -1,3 +1,4 @@
+#-*- perl -*-
 
 package MIME::Charset;
 use 5.005;
@@ -15,7 +16,7 @@ MIME::Charset - Charset Informations for MIME
 Getting charset informations:
 
     $benc = $charset->body_encoding; # e.g. "Q"
-    $cset = $charset->canonical_charset; # e.g. "US-ASCII"
+    $cset = $charset->as_string; # e.g. "US-ASCII"
     $henc = $charset->header_encoding; # e.g. "S"
     $cset = $charset->output_charset; # e.g. "ISO-2022-JP"
 
@@ -24,22 +25,25 @@ Translating text data:
     ($text, $charset, $encoding) =
         $charset->header_encode(
            "\xc9\xc2\xc5\xaa\xc0\xde\xc3\xef\xc5\xaa".
-           "\xc7\xd1\xca\xaa\xbd\xd0\xce\xcf\xb4\xef");
-    # ...returns e.g. (<converted>, "ISO-2022-JP", "B");
+           "\xc7\xd1\xca\xaa\xbd\xd0\xce\xcf\xb4\xef",
+           Charset => 'euc-jp');
+    # ...returns e.g. (<converted>, "ISO-2022-JP", "B").
 
     ($text, $charset, $encoding) =
         $charset->body_encode(
             "Collectioneur path\xe9tiquement ".
-            "\xe9clectique de d\xe9chets");
-    # ...returns e.g. (<original>, "ISO-8859-1", "QUOTED-PRINTABLE");
+            "\xe9clectique de d\xe9chets",
+            Charset => 'latin1');
+    # ...returns e.g. (<original>, "ISO-8859-1", "QUOTED-PRINTABLE").
 
     $len = $charset->encoded_header_len(
-        "Perl\xe8\xa8\x80\xe8\xaa\x9e", "b"); # e.g. 28
+        "Perl\xe8\xa8\x80\xe8\xaa\x9e",
+        Charset => 'utf-8',
+        Encoding => "b");
+    # ...returns e.g. 28.
 
 Manipulating module defaults:
 
-    use MIME::Charset;
-
     MIME::Charset::alias("csEUCKR", "euc-kr");
     MIME::Charset::default("iso-8859-1");
     MIME::Charset::fallback("us-ascii");
@@ -88,8 +92,6 @@ The B<encoding> is that used in MIME to refer to a method of representing
 a body part or a header body as sequence(s) of printable US-ASCII
 characters.
 
-=over 4
-
 =cut
 
 use strict;
@@ -107,7 +109,7 @@ use Exporter;
 		);
 use Carp qw(croak);
 
-use constant USE_ENCODE => ($] >= 5.008001)? 'Encode': '';
+use constant USE_ENCODE => ($] >= 5.008)? 'Encode': '';
 
 my @ENCODE_SUBS = qw(FB_CROAK FB_PERLQQ FB_HTMLCREF FB_XMLCREF
 		     is_utf8 resolve_alias);
@@ -121,7 +123,7 @@ if (USE_ENCODE) {
     }
 }
 
-$VERSION = '1.006';
+$VERSION = '1.007';
 
 ######## Private Attributes ########
 
@@ -198,14 +200,24 @@ my %ENCODERS = (
 		    'ISO-8859-9' => [['cp1254'], ],     # Encode::Byte
 		    'ISO-8859-13'=> [['cp1257'], ],     # Encode::Byte
 		    'GB2312'     => [['cp936'], ],      # Encode::CN
-		    'EUC-JP'     => [['cp51932',        'Encode::EUCJPMS'], ],
+		    'EUC-JP'     => [
+				     ['x-8biteucjpascii',
+				      		       'MIME::Charset::CP932'],
+				     # ['cp51932',        'Encode::EUCJPMS'],
+				    ],
 		    'ISO-2022-JP'=> [
-				     ['cp50220',        'Encode::EUCJPMS'],
+				     ['x-7biteucjpascii',
+				      		       'MIME::Charset::CP932'],
+				     # ['cp50220',      'Encode::EUCJPMS'],
 				     # ['cp50221',      'Encode::EUCJPMS'],
 				     ['iso-2022-jp-ms', 'Encode::ISO2022JPMS'],
 				     ['iso-2022-jp-1'], # Encode::JP (note*)
 				    ],
-		    'SHIFT_JIS'  => [['cp932'], ],      # Encode::JP
+		    'SHIFT_JIS'  => [
+				     # ['x-shifteucjpascii',
+				     # 		       'MIME::Charset::CP932'],
+				     ['cp932'],		# Encode::JP
+				    ],
 		    'EUC-KR'     => [['cp949'], ],      # Encode::KR
 		    'BIG5'       => [
 				     # ['big5plus',     'Encode::HanExtra'],
@@ -237,7 +249,7 @@ my @ESCAPE_SEQS = (
 		   ["\033(J",	"ISO-2022-JP"],	# ditto
 		   ["\033(I",	"ISO-2022-JP"],	# ditto (nonstandard)
 		   ["\033\$(D",	"ISO-2022-JP"],	# RFC 2237 (note*)
-		   # Folloing sequences are less commonly used.
+		   # Following sequences are less commonly used.
 		   ["\033\$)C",	"ISO-2022-KR"],	# RFC 1557
 		   ["\033\$)A",	"ISO-2022-CN"], # RFC 1922
 		   ["\033\$A",	"ISO-2022-CN"], # ditto (nonstandard)
@@ -300,20 +312,22 @@ my $ASCIITRANSRE = qr{
 
 =head2 Constructor
 
+=over
+
 =item $charset = MIME::Charset->new([CHARSET [, OPTS]])
 
 Create charset object.
 
-OPTS may accept following key-value pairs.
+OPTS may accept following key-value pair.
 B<NOTE>:
 When Unicode/multibyte support is disabled (see L<"USE_ENCODE">),
-conversion will not be performed.  So these options do not have any effects.
+conversion will not be performed.  So this option do not have any effects.
 
 =over 4
 
 =item Mapping => MAPTYPE
 
-Specify extended mappings actually used for charset names.
+Whether to extend mappings actually used for charset names or not.
 C<"EXTENDED"> uses extended mappings.
 C<"STANDARD"> uses standardized strict mappings.
 Default is C<"EXTENDED">.
@@ -359,11 +373,12 @@ sub new {
 }
 
 sub _find_encoder($$) {
-    my $charset = uc(shift);
+    my $charset = uc(shift || "");
     return undef unless $charset;
     my $mapping = uc(shift);
     my ($spec, $name, $module, $encoder);
 
+    local($@);
     foreach my $m (('EXTENDED', 'STANDARD')) {
 	next if $m eq 'EXTENDED' and $mapping ne 'EXTENDED';
 	$spec = $ENCODERS{$m}->{$charset};
@@ -381,8 +396,12 @@ sub _find_encoder($$) {
     return Encode::find_encoding($charset);
 }
 
+=back
+
 =head2 Getting Informations of Charsets
 
+=over
+
 =item $charset->body_encoding
 
 =item body_encoding CHARSET
@@ -424,7 +443,7 @@ sub as_string($) {
 
 =item $charset->decoder
 
-Get L<"Encode::Encoding"> object to decode strings by charset.
+Get L<"Encode::Encoding"> object to decode strings to Unicode by charset.
 
 =cut
 
@@ -510,8 +529,12 @@ sub output_charset($) {
     $self->{OutputCharset};
 }
 
+=back
+
 =head2 Translating Text Data
 
+=over
+
 =item $charset->body_encode(STRING [, OPTS])
 
 =item body_encode STRING, CHARSET [, OPTS]
@@ -602,11 +625,48 @@ sub decode($$$;) {
     $self->{Decoder}->decode($s, $check);
 }
 
+=item detect_7bit_charset STRING
+
+Guess 7-bit charset that may encode a string STRING.
+
+=cut
+
+sub detect_7bit_charset($) {
+    return $DEFAULT_CHARSET unless USE_ENCODE;
+    my $s = shift;
+    return $DEFAULT_CHARSET unless $s;
+
+    # Try to detect 7-bit escape sequences.
+    foreach (@ESCAPE_SEQS) {
+	my ($seq, $cset) = @$_;
+	if (index($s, $seq) >= 0) {
+            my $decoder = __PACKAGE__->new($cset);
+            next unless $decoder->{Decoder};
+            eval {
+		my $dummy = $s;
+		$decoder->decode($dummy, FB_CROAK());
+	    };
+	    if ($@) {
+		next;
+	    }
+	    return $decoder->{InputCharset};
+	}
+    }
+
+    # How about HZ, VIQR, UTF-7, ...?
+
+    return $DEFAULT_CHARSET;
+}
+
+sub _detect_7bit_charset {
+    detect_7bit_charset(@_);
+}
+
 =item $charset->encode(STRING [,CHECK])
 
 Encode STRING (Unicode or non-Unicode) using compatible charset recommended
 to be used for messages on Internet (if this module knows it).
-Note that string will be decoded then encoded even if compatible charset
+Note that string will be decoded to Unicode then encoded even if compatible charset
 was equal to original charset.
 
 B<Note>:
@@ -623,7 +683,9 @@ sub encode($$$;) {
     unless (is_utf8($s) or $s =~ /[^\x00-\xFF]/) {
 	$s = $self->{Decoder}->decode($s, ($check & 0x1)? FB_CROAK(): 0);
     }
-    $self->{Encoder}->encode($s, $check);
+    my $enc = $self->{Encoder}->encode($s, $check);
+    Encode::_utf8_off($enc); # workaround for RT #35120
+    $enc;
 }
 
 =item $charset->encoded_header_len(STRING [, ENCODING])
@@ -643,7 +705,7 @@ sub encoded_header_len($$$;) {
     my ($encoding, $s);
     if (ref $self) {
 	$s = shift;
-	$encoding = uc(shift) || $self->{HeaderEncoding};
+	$encoding = uc(shift || $self->{HeaderEncoding});
     } else {
 	$s = $self;
 	$encoding = uc(shift);
@@ -712,8 +774,6 @@ data) and I<encoding scheme> will be C<undef> (should not be encoded).
 I<Charset for output> will be C<"US-ASCII"> if and only if string does not
 contain any non-ASCII bytes.
 
-=back
-
 =cut
 
 sub header_encode {
@@ -765,7 +825,7 @@ sub _text_encode {
 	if ($s =~ $NON7BITRE) {
 	    return ($s, undef);
 	} elsif ($detect7bit ne "NO") {
-	    $charset = __PACKAGE__->new(&_detect_7bit_charset($s));
+	    $charset = __PACKAGE__->new(&detect_7bit_charset($s));
 	} else {
 	    $charset = __PACKAGE__->new($DEFAULT_CHARSET,
 					Mapping => 'STANDARD');
@@ -804,11 +864,13 @@ sub _text_encode {
 						Mapping => 'STANDARD');
 		    # croak unknown charset
 		    croak "unknown charset ``$FALLBACK_CHARSET''"
-			unless $charset->{Decoder};
-		    # charset transformation
+			unless $cset->{Decoder};
+		    # charset translation
+		    $charset = $charset->dup;
 		    $charset->encoder($cset);
 		    $encoded = $s;
 		    $encoded = $charset->encode($encoded, 0);
+		    # replace input & output charsets with fallback charset
 		    $cset->encoder($cset);
 		    $charset = $cset;
 		} else {
@@ -826,55 +888,26 @@ sub _text_encode {
 
     if ($encoded !~ /$NONASCIIRE/) { # maybe ASCII
 	# check ``ASCII transformation'' charsets
-	if ($charset->{OutputCharset} =~ /^($ASCIITRANSRE)$/ and
-	    $encoded =~ /[+~]/) {
+	if ($charset->{OutputCharset} =~ /^($ASCIITRANSRE)$/) {
 	    my $u = $encoded;
-	    if ($charset->encoder) {
+	    if (USE_ENCODE) {
 		$u = $charset->encoder->decode($encoded); # dec. by output
-	    } elsif (!USE_ENCODE) { # workaround for pre-Encode environment
+	    } elsif ($encoded =~ /[+~]/) { # workaround for pre-Encode env.
 		$u = "x$u";
-	    } else { # NOTREACHED
-		croak __PACKAGE__.": bug in _text_encode.  Report developer.";
 	    }
-	    $charset->encoder(__PACKAGE__->new($DEFAULT_CHARSET,
-					       Mapping => 'STANDARD'))
-		if $u eq $encoded;
+	    if ($u eq $encoded) {
+		$charset = $charset->dup;
+		$charset->encoder($DEFAULT_CHARSET);
+	    }
 	} elsif ($charset->{OutputCharset} ne "US-ASCII") {
-	    $charset->encoder(__PACKAGE__->new($DEFAULT_CHARSET,
-					       Mapping => 'STANDARD'));
+	    $charset = $charset->dup;
+	    $charset->encoder($DEFAULT_CHARSET);
 	}
     }
 
     return ($encoded, $charset);
 }
 
-sub _detect_7bit_charset($) {
-    return $DEFAULT_CHARSET unless USE_ENCODE;
-    my $s = shift;
-    return $DEFAULT_CHARSET unless $s;
-
-    # Try to detect 7-bit escape sequences.
-    foreach (@ESCAPE_SEQS) {
-	my ($seq, $cset) = @$_;
-	if (index($s, $seq) >= 0) {
-            my $decoder = __PACKAGE__->new($cset);
-            next unless $decoder and $decoder->{Decoder};
-            eval {
-		my $dummy = $s;
-		$decoder->decode($dummy, FB_CROAK());
-	    };
-	    if ($@) {
-		next;
-	    }
-	    return $decoder->{InputCharset};
-	}
-    }
-
-    # How about HZ, VIQR, UTF-7, ...?
-
-    return $DEFAULT_CHARSET;
-}
-
 =item $charset->undecode(STRING [,CHECK])
 
 Encode Unicode string STRING to byte string by input charset of $charset.
@@ -890,12 +923,16 @@ sub undecode($$$;) {
     my $self = shift;
     my $s = shift;
     my $check = shift || 0;
-    $self->{Decoder}->encode($s, $check);
+    my $enc = $self->{Decoder}->encode($s, $check);
+    Encode::_utf8_off($enc); # workaround for RT #35120
+    $enc;
 }
 
+=back
+
 =head2 Manipulating Module Defaults
 
-=over 4
+=over
 
 =item alias ALIAS [, CHARSET]
 
@@ -1029,20 +1066,28 @@ sub recommended ($;$;$;$) {
     }
 }
 
+=back
+
 =head2 Constants
 
+=over
+
 =item USE_ENCODE
 
 Unicode/multibyte support flag.
 Non-empty string will be set when Unicode and multibyte support is enabled.
-Currently, this flag will be non-empty on Perl 5.8.1 or later and
+Currently, this flag will be non-empty on Perl 5.8.0 or later and
 empty string on earlier versions of Perl.
 
+=back
+
 =head2 Error Handling
 
 L<"body_encode"> and L<"header_encode"> accept following C<Replacement>
 options:
 
+=over
+
 =item C<"DEFAULT">
 
 Put a substitution character in place of a malformed character.
@@ -1097,12 +1142,15 @@ L<http://hatuka.nezumi.nu/repos/MIME-Charset/>.
 
 Multipurpose Internet Mail Extensions (MIME).
 
-=head1 AUTHORS
+=head1 AUTHOR
 
-Copyright (C) 2006-2008 Hatuka*nezumi - IKEDA Soji <hatuka(at)nezumi.nu>.
+Hatuka*nezumi - IKEDA Soji <hatuka(at)nezumi.nu>
 
-All rights reserved.  This program is free software; you can redistribute
-it and/or modify it under the same terms as Perl itself.
+=head1 COPYRIGHT
+
+Copyright (C) 2006-2009 Hatuka*nezumi - IKEDA Soji.
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =cut
 
